@@ -48,9 +48,13 @@ If you have questions concerning this license or the applicable additional terms
 #include "renderer/RenderSystem.h"
 #include "tools/compilers/compiler_public.h"
 #include "tools/compilers/aas/AASFileManager.h"
+#include "tools/edit_public.h"
 
 #include "framework/Common.h"
 
+#include "GameCallbacks_local.h"
+#include "Session_local.h" // DG: For FT_IsDemo/isDemo() hack
+
 #define	MAX_PRINT_MSG_SIZE	4096
 #define MAX_WARNING_LIST	256
 
@@ -68,7 +72,7 @@ typedef enum {
 #endif
 
 struct version_s {
-			version_s( void ) { sprintf( string, "%s.%d%s %s-%s %s %s", ENGINE_VERSION, BUILD_NUMBER, BUILD_DEBUG, BUILD_OS, BUILD_CPU, __DATE__, __TIME__ ); }
+			version_s( void ) { sprintf( string, "%s.%d%s %s-%s %s %s", ENGINE_VERSION, BUILD_NUMBER, BUILD_DEBUG, BUILD_OS, BUILD_CPU, ID__DATE__, ID__TIME__ ); }
 	char	string[256];
 } version;
 
@@ -79,12 +83,8 @@ idCVar com_purgeAll( "com_purgeAll", "0", CVAR_BOOL | CVAR_ARCHIVE | CVAR_SYSTEM
 idCVar com_memoryMarker( "com_memoryMarker", "-1", CVAR_INTEGER | CVAR_SYSTEM | CVAR_INIT, "used as a marker for memory stats" );
 idCVar com_preciseTic( "com_preciseTic", "1", CVAR_BOOL|CVAR_SYSTEM, "run one game tick every async thread update" );
 idCVar com_asyncInput( "com_asyncInput", "0", CVAR_BOOL|CVAR_SYSTEM, "sample input from the async thread" );
-#define ASYNCSOUND_INFO "0: mix sound inline, 1: memory mapped async mix, 2: callback mixing, 3: write async mix"
-#if defined( __unix__ ) && !defined( MACOS_X )
-idCVar com_asyncSound( "com_asyncSound", "3", CVAR_INTEGER|CVAR_SYSTEM|CVAR_ROM, ASYNCSOUND_INFO );
-#else
-idCVar com_asyncSound( "com_asyncSound", "1", CVAR_INTEGER|CVAR_SYSTEM, ASYNCSOUND_INFO, 0, 1 );
-#endif
+#define ASYNCSOUND_INFO "0: mix sound inline, 1 or 3: async update every 16ms 2: async update about every 100ms (original behavior)"
+idCVar com_asyncSound( "com_asyncSound", "1", CVAR_INTEGER|CVAR_SYSTEM, ASYNCSOUND_INFO, 0, 3 );
 idCVar com_forceGenericSIMD( "com_forceGenericSIMD", "0", CVAR_BOOL | CVAR_SYSTEM | CVAR_NOCHEAT, "force generic platform independent SIMD" );
 idCVar com_developer( "developer", "0", CVAR_BOOL|CVAR_SYSTEM|CVAR_NOCHEAT, "developer mode" );
 idCVar com_allowConsole( "com_allowConsole", "0", CVAR_BOOL | CVAR_SYSTEM | CVAR_NOCHEAT, "allow toggling console with the tilde key" );
@@ -162,6 +162,23 @@ public:
 	virtual int					ButtonState( int key );
 	virtual int					KeyState( int key );
 
+	// DG: hack to allow adding callbacks and exporting additional functions without breaking the game ABI
+	//     see Common.h for longer explanation...
+
+	// returns true if setting the callback was successful, else false
+	// When a game DLL is unloaded the callbacks are automatically removed from the Engine
+	// so you usually don't have to worry about that; but you can call this with cb = NULL
+	// and userArg = NULL to remove a callback manually (e.g. if userArg refers to an object you deleted)
+	virtual bool				SetCallback(idCommon::CallbackType cbt, idCommon::FunctionPointer cb, void* userArg);
+
+	// returns true if that function is available in this version of dhewm3
+	// *out_fnptr will be the function (you'll have to cast it probably)
+	// *out_userArg will be an argument you have to pass to the function, if appropriate (else NULL)
+	// NOTE: this doesn't do anything yet, but allows to add ugly mod-specific hacks without breaking the Game interface
+	virtual bool				GetAdditionalFunction(idCommon::FunctionType ft, idCommon::FunctionPointer* out_fnptr, void** out_userArg);
+
+	// DG end
+
 	void						InitGame( void );
 	void						ShutdownGame( bool reloading );
 
@@ -2485,11 +2502,14 @@ void idCommonLocal::SingleAsyncTic( void ) {
 
 	switch ( com_asyncSound.GetInteger() ) {
 		case 1:
-			soundSystem->AsyncUpdate( stat->milliseconds );
-			break;
 		case 3:
+			// DG: these are now used for the new default behavior of "update every async tic (every 16ms)"
 			soundSystem->AsyncUpdateWrite( stat->milliseconds );
 			break;
+		case 2:
+			// DG: use 2 for the old "update only 10x/second" behavior in case anyone likes that..
+			soundSystem->AsyncUpdate( stat->milliseconds );
+			break;
 	}
 
 	// we update com_ticNumber after all the background tasks
@@ -2688,6 +2708,8 @@ void idCommonLocal::UnloadGameDLL( void ) {
 	gameEdit = NULL;
 
 #endif
+
+	gameCallbacks.Reset(); // DG: these callbacks are invalid now because DLL has been unloaded
 }
 
 /*
@@ -2731,6 +2753,9 @@ static unsigned int AsyncTimer(unsigned int interval, void *) {
 	// calculate the next interval to get as close to 60fps as possible
 	unsigned int now = SDL_GetTicks();
 	unsigned int tick = com_ticNumber * USERCMD_MSEC;
+	// FIXME: this is pretty broken and basically always returns 1 because now now is much bigger than tic
+	//        (probably com_tickNumber only starts incrementing a second after engine starts?)
+	//        only reason this works is common->Async() checking again before calling SingleAsyncTic()
 
 	if (now >= tick)
 		return 1;
@@ -2738,12 +2763,122 @@ static unsigned int AsyncTimer(unsigned int interval, void *) {
 	return tick - now;
 }
 
+#ifdef _WIN32
+#include "../sys/win32/win_local.h" // for Conbuf_AppendText()
+#endif // _WIN32
+
+static bool checkForHelp(int argc, char **argv)
+{
+	const char* helpArgs[] = { "--help", "-h", "-help", "-?", "/?" };
+	const int numHelpArgs = sizeof(helpArgs)/sizeof(helpArgs[0]);
+
+	for (int i=0; i<argc; ++i)
+	{
+		const char* arg = argv[i];
+		for (int h=0; h<numHelpArgs; ++h)
+		{
+			if (idStr::Icmp(arg, helpArgs[h]) == 0)
+			{
+#ifdef _WIN32
+				// write it to the Windows-only console window
+				#define WriteString(s) Conbuf_AppendText(s)
+#else // not windows
+				// write it to stdout
+				#define WriteString(s) fputs(s, stdout);
+#endif // _WIN32
+				WriteString(ENGINE_VERSION " - http://dhewm3.org\n");
+				WriteString("Commandline arguments:\n");
+				WriteString("-h or --help: Show this help\n");
+				WriteString("+<command> [command arguments]\n");
+				WriteString("  executes a command (with optional arguments)\n");
+
+				WriteString("\nSome interesting commands:\n");
+				WriteString("+map <map>\n");
+				WriteString("  directly loads the given level, e.g. +map game/hell1\n");
+				WriteString("+exec <config>\n");
+				WriteString("  execute the given config (mainly relevant for dedicated servers)\n");
+				WriteString("+disconnect\n");
+				WriteString("  starts the game, goes directly into main menu without showing\n  logo video\n");
+				WriteString("+connect <host>[:port]\n");
+				WriteString("  directly connect to multiplayer server at given host/port\n");
+				WriteString("  e.g. +connect d3.example.com\n");
+				WriteString("  e.g. +connect d3.example.com:27667\n");
+				WriteString("  e.g. +connect 192.168.0.42:27666\n");
+				WriteString("+set <cvarname> <value>\n");
+				WriteString("  Set the given cvar to the given value, e.g. +set r_fullscreen 0\n");
+				WriteString("+seta <cvarname> <value>\n");
+				WriteString("  like +set, but also makes sure the changed cvar is saved (\"archived\")\n  in a cfg\n");
+
+				WriteString("\nSome interesting cvars:\n");
+				WriteString("+set fs_basepath <gamedata path>\n");
+				WriteString("  set path to your Doom3 game data (the directory base/ is in)\n");
+				WriteString("+set fs_game <modname>\n");
+				WriteString("  start the given addon/mod, e.g. +set fs_game d3xp\n");
+#ifndef ID_DEDICATED
+				WriteString("+set r_fullscreen <0 or 1>\n");
+				WriteString("  start game in windowed (0) or fullscreen (1) mode\n");
+				WriteString("+set r_mode <modenumber>\n");
+				WriteString("  start game in resolution belonging to <modenumber>,\n");
+				WriteString("  use -1 for custom resolutions:\n");
+				WriteString("+set r_customWidth  <size in pixels>\n");
+				WriteString("+set r_customHeight <size in pixels>\n");
+				WriteString("  if r_mode is set to -1, these cvars allow you to specify the\n");
+				WriteString("  width/height of your custom resolution\n");
+#endif // !ID_DEDICATED
+				WriteString("\nSee https://modwiki.dhewm3.org/CVars_%28Doom_3%29 for more cvars\n");
+				WriteString("See https://modwiki.dhewm3.org/Commands_%28Doom_3%29 for more commands\n");
+
+				#undef WriteString
+
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
+#ifdef UINTPTR_MAX // DG: make sure D3_SIZEOFPTR is consistent with reality
+
+#if D3_SIZEOFPTR == 4
+  #if UINTPTR_MAX != 0xFFFFFFFFUL
+    #error "CMake assumes that we're building for a 32bit architecture, but UINTPTR_MAX doesn't match!"
+  #endif
+#elif D3_SIZEOFPTR == 8
+  #if UINTPTR_MAX != 18446744073709551615ULL
+    #error "CMake assumes that we're building for a 64bit architecture, but UINTPTR_MAX doesn't match!"
+  #endif
+#else
+  // Hello future person with a 128bit(?) CPU, I hope the future doesn't suck too much and that you don't still use CMake.
+  // Also, please adapt this check and send a pull request (or whatever way we have to send patches in the future)
+  #error "D3_SIZEOFPTR should really be 4 (for 32bit targets) or 8 (for 64bit targets), what kind of machine is this?!"
+#endif
+
+#endif // UINTPTR_MAX defined
+
 /*
 =================
 idCommonLocal::Init
 =================
 */
 void idCommonLocal::Init( int argc, char **argv ) {
+
+	// in case UINTPTR_MAX isn't defined (or wrong), do a runtime check at startup
+	if ( D3_SIZEOFPTR != sizeof(void*) ) {
+		Sys_Error( "Something went wrong in your build: CMake assumed that sizeof(void*) == %d but in reality it's %d!\n",
+		           (int)D3_SIZEOFPTR, (int)sizeof(void*) );
+	}
+
+	if(checkForHelp(argc, argv))
+	{
+		// game has been started with --help (or similar), usage message has been shown => quit
+#ifdef _WIN32
+		// this enforces that the console window is shown until the user closes it
+		// => checkForHelp() writes to the console window on Windows
+		Sys_Error(".");
+#endif // _WIN32
+		exit(1);
+	}
+
 #ifdef ID_DEDICATED
 	// we want to use the SDL event queue for dedicated servers. That
 	// requires video to be initialized, so we just use the dummy
@@ -2756,7 +2891,7 @@ void idCommonLocal::Init( int argc, char **argv ) {
 #endif
 #endif
 
-	if (SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO))
+	if (SDL_Init(SDL_INIT_TIMER | SDL_INIT_VIDEO | SDL_INIT_JOYSTICK)) // init joystick to work around SDL 2.0.9 bug #4391
 		Sys_Error("Error while initializing SDL: %s", SDL_GetError());
 
 	Sys_InitThreads();
@@ -3107,3 +3242,71 @@ void idCommonLocal::ShutdownGame( bool reloading ) {
 	// shut down the file system
 	fileSystem->Shutdown( reloading );
 }
+
+// DG: below here are hacks to allow adding callbacks and exporting additional functions to the
+//     Game DLL without breaking the ABI. See Common.h for longer explanation...
+
+
+// returns true if setting the callback was successful, else false
+// When a game DLL is unloaded the callbacks are automatically removed from the Engine
+// so you usually don't have to worry about that; but you can call this with cb = NULL
+// and userArg = NULL to remove a callback manually (e.g. if userArg refers to an object you deleted)
+bool idCommonLocal::SetCallback(idCommon::CallbackType cbt, idCommon::FunctionPointer cb, void* userArg)
+{
+	switch(cbt)
+	{
+		case idCommon::CB_ReloadImages:
+			gameCallbacks.reloadImagesCB = (idGameCallbacks::ReloadImagesCallback)cb;
+			gameCallbacks.reloadImagesUserArg = userArg;
+			return true;
+
+		default:
+			Warning("Called idCommon::SetCallback() with unknown CallbackType %d!\n", cbt);
+			return false;
+	}
+}
+
+static bool isDemo(void)
+{
+	return sessLocal.IsDemoVersion();
+}
+
+// returns true if that function is available in this version of dhewm3
+// *out_fnptr will be the function (you'll have to cast it probably)
+// *out_userArg will be an argument you have to pass to the function, if appropriate (else NULL)
+bool idCommonLocal::GetAdditionalFunction(idCommon::FunctionType ft, idCommon::FunctionPointer* out_fnptr, void** out_userArg)
+{
+	if(out_userArg != NULL)
+		*out_userArg = NULL;
+
+	if(out_fnptr == NULL)
+	{
+		Warning("Called idCommon::GetAdditionalFunction() with out_fnptr == NULL!\n");
+		return false;
+	}
+	switch(ft)
+	{
+		case idCommon::FT_IsDemo:
+			*out_fnptr = (idCommon::FunctionPointer)isDemo;
+			// don't set *out_userArg, this function takes no arguments
+			return true;
+
+		default:
+			*out_fnptr = NULL;
+			Warning("Called idCommon::SetCallback() with unknown FunctionType %d!\n", ft);
+			return false;
+	}
+}
+
+
+idGameCallbacks gameCallbacks;
+
+idGameCallbacks::idGameCallbacks()
+: reloadImagesCB(NULL), reloadImagesUserArg(NULL)
+{}
+
+void idGameCallbacks::Reset()
+{
+	reloadImagesCB = NULL;
+	reloadImagesUserArg = NULL;
+}
