@@ -40,6 +40,8 @@ If you have questions concerning this license or the applicable additional terms
 
 #include "renderer/tr_local.h"
 
+#include "framework/GameCallbacks_local.h"
+
 // Vista OpenGL wrapper check
 #ifdef _WIN32
 #include "sys/win32/win_local.h"
@@ -57,6 +59,7 @@ idCVar r_multiSamples( "r_multiSamples", "0", CVAR_RENDERER | CVAR_ARCHIVE | CVA
 idCVar r_mode( "r_mode", "5", CVAR_ARCHIVE | CVAR_RENDERER | CVAR_INTEGER, "video mode number" );
 idCVar r_displayRefresh( "r_displayRefresh", "0", CVAR_RENDERER | CVAR_INTEGER | CVAR_NOCHEAT, "optional display refresh rate option for vid mode", 0.0f, 200.0f );
 idCVar r_fullscreen( "r_fullscreen", "0", CVAR_RENDERER | CVAR_ARCHIVE | CVAR_BOOL, "0 = windowed, 1 = full screen" );
+idCVar r_fullscreenDesktop( "r_fullscreenDesktop", "0", CVAR_RENDERER | CVAR_ARCHIVE | CVAR_BOOL, "0: 'real' fullscreen mode 1: keep resolution 'desktop' fullscreen mode" );
 idCVar r_customWidth( "r_customWidth", "720", CVAR_RENDERER | CVAR_ARCHIVE | CVAR_INTEGER, "custom screen width. set r_mode to -1 to activate" );
 idCVar r_customHeight( "r_customHeight", "486", CVAR_RENDERER | CVAR_ARCHIVE | CVAR_INTEGER, "custom screen height. set r_mode to -1 to activate" );
 idCVar r_singleTriangle( "r_singleTriangle", "0", CVAR_RENDERER | CVAR_BOOL, "only draw a single triangle per primitive" );
@@ -221,6 +224,12 @@ idCVar r_materialOverride( "r_materialOverride", "", CVAR_RENDERER, "overrides a
 
 idCVar r_debugRenderToTexture( "r_debugRenderToTexture", "0", CVAR_RENDERER | CVAR_INTEGER, "" );
 
+// DG: let users disable the "scale menus to 4:3" hack
+idCVar r_scaleMenusTo43( "r_scaleMenusTo43", "1", CVAR_RENDERER | CVAR_ARCHIVE | CVAR_BOOL, "Scale menus, fullscreen videos and PDA to 4:3 aspect ratio" );
+// DG: the fscking patent has finally expired
+idCVar r_useCarmacksReverse( "r_useCarmacksReverse", "1", CVAR_RENDERER | CVAR_ARCHIVE | CVAR_BOOL, "Use Z-Fail (Carmack's Reverse) when rendering shadows" );
+idCVar r_useStencilOpSeparate( "r_useStencilOpSeparate", "1", CVAR_RENDERER | CVAR_ARCHIVE | CVAR_BOOL, "Use glStencilOpSeparate() (if available) when rendering shadows" );
+
 // define qgl functions
 #define QGLPROC(name, rettype, args) rettype (APIENTRYP q##name) args;
 #include "renderer/qgl_proc.h"
@@ -267,6 +276,9 @@ PFNGLPROGRAMLOCALPARAMETER4FVARBPROC	qglProgramLocalParameter4fvARB;
 // GL_EXT_depth_bounds_test
 PFNGLDEPTHBOUNDSEXTPROC                 qglDepthBoundsEXT;
 
+// DG: couldn't find any extension for this, it's supported in GL2.0 and newer, incl OpenGL ES2.0
+PFNGLSTENCILOPSEPARATEPROC qglStencilOpSeparate;
+
 /*
 =================
 R_CheckExtension
@@ -385,6 +397,15 @@ static void R_CheckPortableExtensions( void ) {
 	if ( glConfig.twoSidedStencilAvailable )
 		qglActiveStencilFaceEXT = (PFNGLACTIVESTENCILFACEEXTPROC)GLimp_ExtensionPointer( "glActiveStencilFaceEXT" );
 
+	if( glConfig.glVersion >= 2.0) {
+		common->Printf( "... got GL2.0+ glStencilOpSeparate()\n" );
+		qglStencilOpSeparate = (PFNGLSTENCILOPSEPARATEPROC)GLimp_ExtensionPointer( "glStencilOpSeparate" );
+	} else {
+		// TODO: there was an extension by ATI providing glStencilOpSeparateATI - do we care?
+		common->Printf( "... don't have GL2.0+ glStencilOpSeparate()\n" );
+		qglStencilOpSeparate = NULL;
+	}
+
 	// ARB_vertex_buffer_object
 	glConfig.ARBVertexBufferObjectAvailable = R_CheckExtension( "GL_ARB_vertex_buffer_object" );
 	if(glConfig.ARBVertexBufferObjectAvailable) {
@@ -482,8 +503,11 @@ vidmode_t r_vidModes[] = {
 	{ "Mode 19: 3200x2400",		3200,	2400 },
 	{ "Mode 20: 3840x2160",		3840,   2160 },
 	{ "Mode 21: 4096x2304",		4096,   2304 },
+	{ "Mode 22: 2880x1800",		2880,   1800 },
+	{ "Mode 23: 2560x1440",		2560,   1440 },
 };
-static int	s_numVidModes = ( sizeof( r_vidModes ) / sizeof( r_vidModes[0] ) );
+// DG: made this an enum so even stupid compilers accept it as array length below
+enum {	s_numVidModes = sizeof( r_vidModes ) / sizeof( r_vidModes[0] ) };
 
 static bool R_GetModeInfo( int *width, int *height, int mode ) {
 	vidmode_t	*vm;
@@ -513,21 +537,60 @@ static bool R_GetModeInfo( int *width, int *height, int mode ) {
 	return true;
 }
 
+// DG: I added all this vidModeInfoPtr stuff, so I can have a second list of vidmodes
+//     that are sorted (by width, height), instead of just r_mode index.
+//     That way I can add modes without breaking r_mode, but still display them
+//     sorted in the menu.
+
+struct vidModePtr {
+	vidmode_t* vidMode;
+	int modeIndex;
+};
+
+static vidModePtr sortedVidModes[s_numVidModes];
+
+static int vidModeCmp(const void* vm1, const void* vm2)
+{
+	const vidModePtr* v1 = static_cast<const vidModePtr*>(vm1);
+	const vidModePtr* v2 = static_cast<const vidModePtr*>(vm2);
+
+	// sort primarily by width, secondarily by height
+	int wdiff = v1->vidMode->width - v2->vidMode->width;
+	return (wdiff != 0) ? wdiff : (v1->vidMode->height - v2->vidMode->height);
+}
+
+static void initSortedVidModes()
+{
+	if(sortedVidModes[0].vidMode != NULL)
+	{
+		// already initialized
+		return;
+	}
+
+	for(int i=0; i<s_numVidModes; ++i)
+	{
+		sortedVidModes[i].modeIndex = i;
+		sortedVidModes[i].vidMode = &r_vidModes[i];
+	}
+
+	qsort(sortedVidModes, s_numVidModes, sizeof(vidModePtr), vidModeCmp);
+}
+
 // DG: the following two functions are part of a horrible hack in ChoiceWindow.cpp
 //     to overwrite the default resolution list in the system options menu
 
 // "r_custom*;640x480;800x600;1024x768;..."
-idStr R_GetVidModeListString()
+idStr R_GetVidModeListString(bool addCustom)
 {
-	idStr ret = "r_custom*";
-	// for some reason, modes 0-2 are not used. maybe too small for GUI?
-	for(int mode=3; mode<s_numVidModes; ++mode)
+	idStr ret = addCustom ? "r_custom*" : "";
+
+	for(int i=0; i<s_numVidModes; ++i)
 	{
-		int w, h;
-		if(R_GetModeInfo(&w, &h, mode))
+		// for some reason, modes 0-2 are not used. maybe too small for GUI?
+		if(sortedVidModes[i].modeIndex >= 3 && sortedVidModes[i].vidMode != NULL)
 		{
 			idStr modeStr;
-			sprintf(modeStr, ";%dx%d", w, h);
+			sprintf(modeStr, ";%dx%d", sortedVidModes[i].vidMode->width, sortedVidModes[i].vidMode->height);
 			ret += modeStr;
 		}
 	}
@@ -535,13 +598,17 @@ idStr R_GetVidModeListString()
 }
 
 // r_mode values for resolutions from R_GetVidModeListString(): "-1;3;4;5;..."
-idStr R_GetVidModeValsString()
+idStr R_GetVidModeValsString(bool addCustom)
 {
-	idStr ret =  "-1"; // for custom resolutions using r_customWidth/r_customHeight
-	for(int mode=3; mode<s_numVidModes; ++mode)
+	idStr ret = addCustom ? "-1" : ""; // for custom resolutions using r_customWidth/r_customHeight
+	for(int i=0; i<s_numVidModes; ++i)
 	{
-		ret += ";";
-		ret += mode;
+		// for some reason, modes 0-2 are not used. maybe too small for GUI?
+		if(sortedVidModes[i].modeIndex >= 3 && sortedVidModes[i].vidMode != NULL)
+		{
+			ret += ";";
+			ret += sortedVidModes[i].modeIndex;
+		}
 	}
 	return ret;
 }
@@ -579,6 +646,8 @@ void R_InitOpenGL( void ) {
 	tr.viewportOffset[0] = 0;
 	tr.viewportOffset[1] = 0;
 
+	initSortedVidModes();
+
 	//
 	// initialize OS specific portions of the renderSystem
 	//
@@ -605,7 +674,7 @@ void R_InitOpenGL( void ) {
 		// if we failed, set everything back to "safe mode"
 		// and try again
 		r_mode.SetInteger( 3 );
-		r_fullscreen.SetInteger( 1 );
+		r_fullscreen.SetInteger( 0 );
 		r_displayRefresh.SetInteger( 0 );
 		r_multiSamples.SetInteger( 0 );
 	}
@@ -1261,7 +1330,7 @@ thousands of shots
 void R_ScreenshotFilename( int &lastNumber, const char *base, idStr &fileName ) {
 	int	a,b,c,d, e;
 
-	bool restrict = cvarSystem->GetCVarBool( "fs_restrict" );
+	bool fsrestrict = cvarSystem->GetCVarBool( "fs_restrict" );
 	cvarSystem->SetCVarBool( "fs_restrict", false );
 
 	lastNumber++;
@@ -1291,7 +1360,7 @@ void R_ScreenshotFilename( int &lastNumber, const char *base, idStr &fileName )
 		}
 		// check again...
 	}
-	cvarSystem->SetCVarBool( "fs_restrict", restrict );
+	cvarSystem->SetCVarBool( "fs_restrict", fsrestrict );
 }
 
 /*
@@ -1794,6 +1863,12 @@ void R_VidRestart_f( const idCmdArgs &args ) {
 		return;
 	}
 
+	// DG: notify the game DLL about the reloadImages and vid_restart commands
+	if(gameCallbacks.reloadImagesCB != NULL)
+	{
+		gameCallbacks.reloadImagesCB(gameCallbacks.reloadImagesUserArg, args);
+	}
+
 	bool full = true;
 	bool forceWindow = false;
 	for ( int i = 1 ; i < args.Argc() ; i++ ) {
@@ -2069,6 +2144,8 @@ void idRenderSystemLocal::Init( void ) {
 	identitySpace.modelMatrix[0*4+0] = 1.0f;
 	identitySpace.modelMatrix[1*4+1] = 1.0f;
 	identitySpace.modelMatrix[2*4+2] = 1.0f;
+
+	origWidth = origHeight = 0; // DG: for resetting width/height in EndFrame()
 }
 
 /*
