@@ -26,8 +26,8 @@ If you have questions concerning this license or the applicable additional terms
 ===========================================================================
 */
 
-#include "../../idlib/precompiled.h"
-#pragma hdrstop
+#include "tools/edit_gui_common.h"
+
 
 #include "qe3.h"
 #include "Radiant.h"
@@ -445,6 +445,7 @@ void idGLDrawableMaterial::draw(int x, int y, int w, int h) {
 		}
 
 		renderView_t	refdef;
+		
 		// render it
 		renderSystem->BeginFrame(w, h);
 		memset( &refdef, 0, sizeof( refdef ) );
