@@ -56,6 +56,7 @@ idCVar idWindow::gui_debug( "gui_debug", "0", CVAR_GUI | CVAR_BOOL, "" );
 idCVar idWindow::gui_edit( "gui_edit", "0", CVAR_GUI | CVAR_BOOL, "" );
 
 extern idCVar r_skipGuiShaders;		// 1 = don't render any gui elements on surfaces
+extern idCVar r_scaleMenusTo43;
 
 //  made RegisterVars a member of idWindow
 const idRegEntry idWindow::RegisterVars[] = {
@@ -250,6 +251,11 @@ void idWindow::CleanUp() {
 	// Cleanup the named events
 	namedEvents.DeleteContents(true);
 
+	// Cleanup the operations and update vars
+	// (if it is not fixed, orphane register references are possible)
+	ops.Clear();
+	updateVars.Clear();
+
 	drawWindows.Clear();
 	children.DeleteContents(true);
 	definedVars.DeleteContents(true);
@@ -1207,6 +1213,19 @@ void idWindow::Redraw(float x, float y) {
 		return;
 	}
 
+	// DG: allow scaling menus to 4:3
+	bool fixupFor43 = false;
+	if ( flags & WIN_DESKTOP ) {
+		// only scale desktop windows (will automatically scale its sub-windows)
+		// that EITHER have the scaleto43 flag set OR are fullscreen menus and r_scaleMenusTo43 is 1
+		if( (flags & WIN_SCALETO43) ||
+			((flags & WIN_MENUGUI) && r_scaleMenusTo43.GetBool()) )
+		{
+			fixupFor43 = true;
+			dc->SetMenuScaleFix(true);
+		}
+	}
+
 	if ( flags & WIN_SHOWTIME ) {
 		dc->DrawText(va(" %0.1f seconds\n%s", (float)(time - timeLine) / 1000, gui->State().GetString("name")), 0.35f, 0, dc->colorWhite, idRectangle(100, 0, 80, 80), false);
 	}
@@ -1219,6 +1238,9 @@ void idWindow::Redraw(float x, float y) {
 	}
 
 	if (!visible) {
+		if (fixupFor43) { // DG: gotta reset that before returning this function
+			dc->SetMenuScaleFix(false);
+		}
 		return;
 	}
 
@@ -1287,6 +1309,10 @@ void idWindow::Redraw(float x, float y) {
 		dc->EnableClipping(true);
 	}
 
+	if (fixupFor43) { // DG: gotta reset that before returning this function
+		dc->SetMenuScaleFix(false);
+	}
+
 	drawRect.Offset(-x, -y);
 	clientRect.Offset(-x, -y);
 	textRect.Offset(-x, -y);
@@ -1925,6 +1951,15 @@ bool idWindow::ParseInternalVar(const char *_name, idParser *src) {
 		}
 		return true;
 	}
+	// DG: added this window flag for Windows that should be scaled to 4:3
+	//     (with "empty" bars left/right or above/below)
+	if (idStr::Icmp(_name, "scaleto43") == 0) {
+		if ( src->ParseBool() ) {
+			flags |= WIN_SCALETO43;
+		}
+		return true;
+	}
+	// DG end
 	if (idStr::Icmp(_name, "forceaspectwidth") == 0) {
 		forceAspectWidth = src->ParseFloat();
 		return true;
